package de.hhhammer.playtime.ng;

import de.hhhammer.playtime.ng.persistence.PlayerTimeDB;
import de.hhhammer.playtime.ng.persistence.ReadPlayerException;
import de.hhhammer.playtime.ng.persistence.WritePlayerException;
import de.hhhammer.playtime.ng.player.PlayTimePlayer;

import java.time.Clock;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.UUID;

public class PlayerTimeService {
    private final Clock clock;
    private final PlayerTimeDB playerTimeDB;
    private final PlaytimeCalculator playtimeCalculator;

    public PlayerTimeService(Clock clock, PlayerTimeDB playerTimeDB, PlaytimeCalculator playtimeCalculator) {
        this.clock = clock;
        this.playerTimeDB = playerTimeDB;
        this.playtimeCalculator = playtimeCalculator;
    }

    public PlayerTimeService(PlayerTimeDB playerTimeDB, PlaytimeCalculator playtimeCalculator) {
        this(Clock.systemDefaultZone(), playerTimeDB, playtimeCalculator);
    }

    public void playerJoin(UUID uuid, String playerName) {
        try {
            PlayTimePlayer player = playerTimeDB.findById(uuid)
                    .map(foundPlayer -> new PlayTimePlayer(foundPlayer.uuid(), foundPlayer.name(), foundPlayer.playtime(), LocalDateTime.now(clock)))
                    .orElse(new PlayTimePlayer(uuid, playerName, Duration.ZERO, LocalDateTime.now(clock)));
            playerTimeDB.flush(player);
        } catch (ReadPlayerException | WritePlayerException e) {
            throw new RuntimeException(e);
        }
    }

    public void updatePlayTime(UUID uuid) {
        try {
            playerTimeDB.findById(uuid)
                    .map(player -> {
                        Duration playtime = playtimeCalculator.calculatePlaytime(player);
                        return new PlayTimePlayer(player.uuid(), player.name(), playtime, player.joinTime(), player.saveTime());
                    }).ifPresent(player -> {
                        try {
                            playerTimeDB.flush(player);
                        } catch (WritePlayerException e) {
                            throw new RuntimeException(e);
                        }
                    });
        } catch (ReadPlayerException e) {

        }
    }

    /**
     * timePlayed(uuid: UUID): Duration
     * getTopPlayers(): List<Player>
     * timeToString(duration: Duration): String
     */
}
