package de.hhhammer.playtime.ng;

import de.hhhammer.playtime.ng.player.Player;

import java.time.Clock;
import java.time.Duration;
import java.time.LocalDateTime;

public class PlaytimeCalculator {

    private final Clock clock;

    public PlaytimeCalculator(Clock clock) {
        this.clock = clock;
    }

    public PlaytimeCalculator() {
        this(Clock.systemDefaultZone());
    }

    public Duration calculatePlaytime(Player player) {
        Duration playTime = timePassedBetweenSaveOrJoin(player);
        return Duration.from(player.playtime()).plus(playTime);
    }

    private Duration timePassedBetweenSaveOrJoin(Player player) {
        if (player.saveTime().isPresent() && Duration.between(player.saveTime().get(), player.joinTime()).isNegative()) {
            return Duration.between(player.saveTime().get(), LocalDateTime.now(clock));
        }
        return Duration.between(player.joinTime(), LocalDateTime.now(clock));
    }
}
