package de.hhhammer.playtime.ng.player;

import de.hhhammer.playtime.ng.serialization.deserializer.Deserializer;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAccessor;
import java.util.Optional;
import java.util.UUID;

public final class DeserializationPlayer implements Player {
    private final Deserializer deserializer;
    private final DateTimeFormatter dateTimeFormatter;

    public DeserializationPlayer(final Deserializer deserializer, final DateTimeFormatter dateTimeFormatter) {
        this.deserializer = deserializer;
        this.dateTimeFormatter = dateTimeFormatter;
    }

    @Override
    public UUID uuid() {
        final String uuid = deserializer.getValueForKey("uuid");
        return UUID.fromString(uuid);
    }

    @Override
    public String name() {
        return deserializer.getValueForKey("name");
    }

    @Override
    public Duration playtime() {
        final String[] time = deserializer.getValueForKey("playtime").split(":");

        return Duration.ofHours(Integer.parseInt(time[0]))
                .plusMinutes(Integer.parseInt(time[1]))
                .plusSeconds(Integer.parseInt(time[2]));
    }

    @Override
    public LocalDateTime joinTime() {
        final String joinTime = deserializer.getValueForKey("joinTime");
        final TemporalAccessor joinTimeFormatted = dateTimeFormatter.parse(joinTime);
        return LocalDateTime.from(joinTimeFormatted);
    }

    @Override
    public Optional<LocalDateTime> saveTime() {
        return Optional.empty();
    }
}
