package de.hhhammer.playtime.ng.player;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Optional;
import java.util.UUID;

public final class PlayTimePlayer implements SettableSaveTimePlayer {
    private final Player player;

    public PlayTimePlayer(final Player player) {
        this.player = player;
    }

    public PlayTimePlayer(final UUID uuid, final String name, final Duration playtime, final LocalDateTime joinTime, final Optional<LocalDateTime> saveTime) {
        this(new PlayerRecord(uuid, name, playtime, joinTime, saveTime));
    }
    public PlayTimePlayer(final UUID uuid, final String name, final Duration playtime, final LocalDateTime joinTime, final LocalDateTime saveTime) {
        this(new PlayerRecord(uuid, name, playtime, joinTime, Optional.of(saveTime)));
    }

    public PlayTimePlayer(final UUID uuid, final String name, final Duration playtime, final LocalDateTime joinTime) {
        this(new PlayerRecord(uuid, name, playtime, joinTime, Optional.empty()));
    }

    @Override
    public Player withSaveTime(final LocalDateTime saveTime) {
        return new PlayerRecord(uuid(), name(), playtime(), joinTime(), Optional.of(saveTime));
    }

    @Override
    public UUID uuid() {
        return player.uuid();
    }

    @Override
    public String name() {
        return player.name();
    }

    @Override
    public Duration playtime() {
        return player.playtime();
    }

    @Override
    public LocalDateTime joinTime() {
        return player.joinTime();
    }

    @Override
    public Optional<LocalDateTime> saveTime() {
        return player.saveTime();
    }

    private record PlayerRecord(UUID uuid, String name, Duration playtime, LocalDateTime joinTime,
                                Optional<LocalDateTime> saveTime) implements Player {
    }

}
