package de.hhhammer.playtime.ng.serialization.deserializer;

import java.util.HashMap;
import java.util.Map;

public final class JsonDeserializer implements Deserializer {
    private final String serializedPlayer;

    public JsonDeserializer(final String serializedPlayer) {
        this.serializedPlayer = serializedPlayer;
    }

    @Override
    public String getValueForKey(final String field) {
        final String withoutObjectBraces = removeStartingAndEndingObjectCurlyBraces();
        final String withoutControlCharacters = withoutObjectBraces.replaceAll("\n", "")
                .replaceAll("\t", "");
        final String withoutQuotes = withoutControlCharacters.replaceAll("\"", "");
        final String[] keyValuePairs = withoutQuotes.split(",");
        final Map<String, String> map = createMap(keyValuePairs);
        return map.get(field);
    }

    private Map<String, String> createMap(final String[] keyValuePairs) {
        final var map = new HashMap<String, String>();
        for (final var keyValuePair : keyValuePairs) {
            final int indexOfKeyValueDelimiter = keyValuePair.indexOf(":");
            final String key = keyValuePair.substring(0, indexOfKeyValueDelimiter);
            // we need to add 1 to skip delimiter character
            final String value = keyValuePair.substring(indexOfKeyValueDelimiter + 1);
            map.put(key.trim(), value.trim());
        }

        return map;
    }

    private String removeStartingAndEndingObjectCurlyBraces() {
        final String withoutOpeningBraces = serializedPlayer.replaceFirst("\\{", "");
        final String reverseWithoutClosingBraces = new StringBuffer(withoutOpeningBraces).reverse()
                .toString()
                .replaceFirst("}", "");
        return new StringBuffer(reverseWithoutClosingBraces).reverse().toString();
    }
}
