package de.hhhammer.playtime.ng.serialization.serializer;

import de.hhhammer.playtime.ng.player.Player;

import java.time.Duration;
import java.time.format.DateTimeFormatter;
import java.util.UUID;

public final class JsonSerializer implements Serializer {
    private final Player player;
    private final DateTimeFormatter dateTimeFormatter;

    public JsonSerializer(final Player player, final DateTimeFormatter dateTimeFormatter) {
        this.player = player;
        this.dateTimeFormatter = dateTimeFormatter;
    }

    @Override
    public String serialize() {
        return "{\n"
                + "\t\"uuid\": \"" + player.uuid() + "\",\n"
                + "\t\"name\": \"" + player.name() + "\",\n"
                + "\t\"playtime\": \"" + formattedDuration(player.playtime()) + "\",\n"
                + "\t\"joinTime\": \"" + player.joinTime().format(dateTimeFormatter) + "\"\n"
                + "}\n";
    }

    @Override
    public UUID uuid() {
        return player.uuid();
    }


    private String formattedDuration(Duration duration) {
        final long daysInHours = duration.toDays() * 24;
        final long unformattedHours = daysInHours + duration.toHoursPart();
        final String hours = unformattedHours < 10 ?
                "0" + unformattedHours : String.valueOf(unformattedHours);
        final String minutes = duration.toMinutesPart() < 10 ?
                "0" + duration.toMinutesPart() : String.valueOf(duration.toMinutesPart());
        final String seconds = duration.toSecondsPart() < 10 ?
                "0" + duration.toSecondsPart() : String.valueOf(duration.toSecondsPart());
        return hours + ":" + minutes + ":" + seconds;
    }
}
