package de.hhhammer.playtime.ng;

import de.hhhammer.playtime.ng.persistence.PlayerTimeDB;
import de.hhhammer.playtime.ng.persistence.ReadPlayerException;
import de.hhhammer.playtime.ng.persistence.WritePlayerException;
import de.hhhammer.playtime.ng.player.PlayTimePlayer;
import de.hhhammer.playtime.ng.player.Player;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.time.*;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
class PlayerTimeServiceTest {
    private final Clock clock = Clock.fixed(Instant.now(), ZoneId.systemDefault());

    @Nested
    class PlayerJoinTest {
        @Mock
        private PlayerTimeDB playerTimeDB;
        @Mock
        private PlaytimeCalculator playtimeCalculator;
        @Captor
        ArgumentCaptor<Player> playerCaptor;

        private PlayerTimeService playerTimeService;

        @BeforeEach
        void setup() {
            playerTimeService = new PlayerTimeService(clock, playerTimeDB, playtimeCalculator);
        }

        @Test
        void shouldSetJoinTimeOnFoundPlayer() throws ReadPlayerException, WritePlayerException {
            var uuid = UUID.randomUUID();
            when(playerTimeDB.findById(uuid)).thenReturn(Optional.of(new PlayTimePlayer(uuid, "username", Duration.ZERO, LocalDateTime.now(clock).minusMinutes(5))));

            playerTimeService.playerJoin(uuid, "someusername");
            verify(playerTimeDB).flush(playerCaptor.capture());
            assertEquals(LocalDateTime.now(clock), playerCaptor.getValue().joinTime());
        }

        @Test
        void shouldSetJoinTimeOnNotFoundPlayer() throws ReadPlayerException, WritePlayerException {
            var uuid = UUID.randomUUID();
            when(playerTimeDB.findById(uuid)).thenReturn(Optional.empty());

            playerTimeService.playerJoin(uuid, "someusername");
            verify(playerTimeDB).flush(playerCaptor.capture());
            assertEquals(LocalDateTime.now(clock), playerCaptor.getValue().joinTime());
        }

        @Test
        void shouldHavePlayTimeOnFoundPlayer() throws ReadPlayerException, WritePlayerException {
            var uuid = UUID.randomUUID();
            when(playerTimeDB.findById(uuid)).thenReturn(Optional.of(new PlayTimePlayer(uuid, "username", Duration.ofHours(1), LocalDateTime.now(clock).minusMinutes(5))));

            playerTimeService.playerJoin(uuid, "someusername");
            verify(playerTimeDB).flush(playerCaptor.capture());
            assertEquals(1, playerCaptor.getValue().playtime().toHours());
        }
    }

}