package de.hhhammer.playtime.ng;

import de.hhhammer.playtime.ng.player.PlayTimePlayer;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import java.time.*;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;

class PlaytimeCalculatorTest {

    @Nested
    class CalculatePlaytimeTest {
        private final Clock clock = Clock.fixed(Instant.parse("2018-08-22T10:00:00Z"), ZoneId.systemDefault());

        @Test
        void shouldReturnTimeBetweenJoinAndNowWhenNotSaved() {
            var playtimeCalculator = new PlaytimeCalculator(clock);
            var player = new PlayTimePlayer(UUID.randomUUID(), "Name", Duration.ofHours(1), LocalDateTime.now(clock).minusMinutes(5));

            Duration playtime = playtimeCalculator.calculatePlaytime(player);
            assertEquals(Duration.ofHours(1)
                            .plus(Duration.ofMinutes(5)),
                    playtime);
        }

        @Test
        void shouldReturnTimeBetweenJoinAndNowWhenSavedBeforeJoin() {
            var playtimeCalculator = new PlaytimeCalculator(clock);
            var player = new PlayTimePlayer(UUID.randomUUID(), "Name", Duration.ofHours(1), LocalDateTime.now(clock).minusMinutes(5));

            Duration playtime = playtimeCalculator.calculatePlaytime(player);
            assertEquals(Duration.ofHours(1)
                            .plus(Duration.ofMinutes(5)),
                    playtime);
        }

        @Test
        void shouldReturnTimeBetweenSaveAndNowWhenSavedAfterJoin() {
            var playtimeCalculator = new PlaytimeCalculator(clock);
            var player = new PlayTimePlayer(UUID.randomUUID(), "Name", Duration.ofHours(1), LocalDateTime.now(clock).minusMinutes(5), LocalDateTime.now(clock).minusMinutes(1));

            Duration playtime = playtimeCalculator.calculatePlaytime(player);
            assertEquals(Duration.ofHours(1)
                            .plus(Duration.ofMinutes(1)),
                    playtime);
        }
    }
}