package de.hhhammer.playtime.ng.player;

import de.hhhammer.playtime.ng.serialization.deserializer.Deserializer;
import org.junit.jupiter.api.Test;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.Month;
import java.time.format.DateTimeFormatter;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class DeserializationPlayerTest {
    private final DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ISO_DATE_TIME;
    private final Deserializer deserializer = mock(Deserializer.class);

    @Test
    void shouldParseUUID() {
        when(deserializer.getValueForKey(anyString())).thenReturn("08f45958-27cc-4d56-b4ea-016cf66c0502");
        final var jsonPlayer = new DeserializationPlayer(deserializer, dateTimeFormatter);

        assertEquals(UUID.fromString("08f45958-27cc-4d56-b4ea-016cf66c0502"), jsonPlayer.uuid());
    }

    @Test
    void shouldParseName() {
        when(deserializer.getValueForKey(anyString())).thenReturn("name");
        final var jsonPlayer = new DeserializationPlayer(deserializer, dateTimeFormatter);

        assertEquals("name", jsonPlayer.name());
    }

    @Test
    void shouldParsePlaytime() {
        when(deserializer.getValueForKey(anyString())).thenReturn("00:01:00");
        final var jsonPlayer = new DeserializationPlayer(deserializer, dateTimeFormatter);

        assertEquals(Duration.ofMinutes(1), jsonPlayer.playtime());
    }

    @Test
    void shouldParseJoinTime() {
        final var joinTime = LocalDateTime.of(2022, Month.JULY, 29, 12, 17, 27, 668767674);
        when(deserializer.getValueForKey(anyString())).thenReturn("2022-07-29T12:17:27.668767674");
        final var jsonPlayer = new DeserializationPlayer(deserializer, dateTimeFormatter);

        assertEquals(joinTime, jsonPlayer.joinTime());
    }

    @Test
    void shouldReturnEmptySaveTime() {
        final var jsonPlayer = new DeserializationPlayer(deserializer, dateTimeFormatter);

        assertTrue(jsonPlayer.saveTime().isEmpty());
    }


}