package de.hhhammer.playtime.ng.player;

import de.hhhammer.playtime.ng.serialization.serializer.JsonSerializer;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.Month;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;

class PlayTimePlayerTest implements PlayerTestHelper {

    @Nested
    class SerializeTest {

        @Test
        void shouldProduceCorrectFormat() {
            final var joinTime = LocalDateTime.of(2022, Month.JULY, 29, 12, 17, 27, 668767674);
            final Player player = new PlayTimePlayer(UUID.fromString("08f45958-27cc-4d56-b4ea-016cf66c0502"), "name", Duration.of(1, ChronoUnit.MINUTES), joinTime);

            final var expected = """
                    {
                    \t"uuid": "08f45958-27cc-4d56-b4ea-016cf66c0502",
                    \t"name": "name",
                    \t"playtime": "00:01:00",
                    \t"joinTime": "2022-07-29T12:17:27.668767674"
                    }
                    """;
            final var actual = new JsonSerializer(player, DateTimeFormatter.ISO_DATE_TIME).serialize();

            assertEquals(expected, actual);
        }
    }

}