package de.hhhammer.playtime.ng.serialization.deserializer;

import org.junit.jupiter.api.Test;

import java.time.LocalDateTime;
import java.time.Month;

import static org.junit.jupiter.api.Assertions.assertEquals;

class JsonDeserializerTest {

    @Test
    void shouldParseValidJsonWithDashValue() {
        final var json = """
                {
                \t"uuid": "08f45958-27cc-4d56-b4ea-016cf66c0502",
                }
                """;
        final var jsonDeserializer = new JsonDeserializer(json);

        assertEquals("08f45958-27cc-4d56-b4ea-016cf66c0502", jsonDeserializer.getValueForKey("uuid"));
    }

    @Test
    void shouldParseValidJsonWithStringValue() {
        final var json = """
                {
                \t"name": "name value",
                }
                """;
        final var jsonDeserializer = new JsonDeserializer(json);

        assertEquals("name value", jsonDeserializer.getValueForKey("name"));
    }

    @Test
    void shouldParseValidJsonWithDoublePoints() {
        final var json = """
                {
                \t"playtime": "00:01:00",
                }
                """;
        final var jsonDeserializer = new JsonDeserializer(json);

        assertEquals("00:01:00", jsonDeserializer.getValueForKey("playtime"));
    }

    @Test
    void shouldParseValidJsonWithDateValue() {
        final var joinTime = LocalDateTime.of(2022, Month.JULY, 29, 12, 17, 27, 668767674);
        final var json = """
                {
                \t"joinTime": "2022-07-29T12:17:27.668767674",
                }
                """;
        final var jsonDeserializer = new JsonDeserializer(json);

        assertEquals("2022-07-29T12:17:27.668767674", jsonDeserializer.getValueForKey("joinTime"));
    }

}
